@@ -14,6 +14,27 @@ pub mod macros {
         };
     }
     use std::any::Any;
+    pub fn is_i8(s: &dyn Any) -> bool {
+        if s.is::<i8>() {
+            return true;
+        } else {
+            return false;
+        }
+    }
+    pub fn is_i16(s: &dyn Any) -> bool {
+        if s.is::<i16>() {
+            return true;
+        } else {
+            return false;
+        }
+    }
+    pub fn is_i32(s: &dyn Any) -> bool {
+        if s.is::<i32>() {
+            return true;
+        } else {
+            return false;
+        }
+    }
     pub fn is_i64(s: &dyn Any) -> bool {
         if s.is::<i64>() {
             return true;
@@ -21,6 +42,20 @@ pub mod macros {
             return false;
         }
     }
+    pub fn is_i128(s: &dyn Any) -> bool {
+        if s.is::<i64>() {
+            return true;
+        } else {
+            return false;
+        }
+    }
+    pub fn is_f32(s: &dyn Any) -> bool {
+        if s.is::<f32>() {
+            return true;
+        } else {
+            return false;
+        }
+    }
     pub fn is_f64(s: &dyn Any) -> bool {
         if s.is::<f64>() {
             return true;
@@ -54,7 +89,7 @@ pub mod macros {
     #[macro_export]
     macro_rules! powf {
         ($a:expr,$b:expr) => {
-            pow($a, $b)
+            f64::powf($a, $b)
         };
     }
     /// arg!() Example
@@ -100,11 +135,30 @@ pub mod macros {
                 .collect::<Vec<String>>()
         };
     }
+    /// read_csv!() Example
+    /// read .csv file and return Vec<Vec<String>>
+    /// ```rust
+    /// let s = read_csv!("./data.csv");
+    /// println!("{:?}",s);
+    /// ```
+    #[macro_export]
+    macro_rules! read_csv {
+        ($path:expr) => {{
+            let data = std::fs::read($path).unwrap();
+            let data_vec = split_to_vec!(&String::from_utf8_lossy(&data), "\r\n");
+            let data_each_vec = data_vec
+                .iter()
+                .map(|s| split_to_vec!(s, ","))
+                .collect::<Vec<_>>();
+            data_each_vec
+        }};
+    }
     pub fn run() {
         // println!("{:?}", powf!(4., 2.));
         // println!("{:?}", split_to_vec!("dnrops.gitee.io", "."));
         // println!("{:?}", args!()); //get argument Vec<String>
         // println!("{:?}", input!()); //get input from treminal to String
+        println!("{:?}", read_csv!("./data.csv"));
     }
 }
 
